#ifndef __TRIANGLEMESH__FOR_MT__
#define __TRIANGLEMESH__FOR_MT__

#include <vector>
#include <i3d/image3d.h>
#include "../src/params.h"

/**
 * This is simple, minimalistically compatible (somewhat at the syntax level (*))
 * class alternative to the main MotionTracking class ActiveMesh.
 *
 * (*) It uses standard container (std::vector) instead of those of MotionTracking (MArray).
 * Data from both containers, however, can be accessed in a syntactically the same fashion,
 * i.e., with pointers and []. Modifying lengths of the containers differ.
 *
 * (*) Also the Vector3F is different from the MotionTracking one, yet they are similar.
 *
 * It is meant for rapid trying/debugging of basic "mesh-processing" mesh-not-modifying algorithms.
 */
class ActiveMesh
{
  public:
	std::vector<Vector3FC> Pos;        //list of mesh vertices
	std::vector<long unsigned int> ID; //list of triangles organized in tripplets of indices into the Pos array of vertices;
                                      //the array length should be 3x the number of triangles in the mesh
	std::vector<Vector3F> norm;        //list of vectors that are normal/orthogonal to the planes given by the individual triangles


	/**
	 * Determine coeffs that represent the local surface around vertex \e vertexID.
	 *
	 * Any point [x,y,z] on the curve satisfies the equation:
	 * Scalar product of \e coeffs and [1,x,y,z,xy,xz,yz,x^2,y^2,z^2] = 0.
	 * This also explains the order (and meaning) of the \e coeffs array.
	 * Order was adopted from [1], eq (2), page 3.
	 *
	 * The algorithm has been implemented according to
	 * the "instructions" [2], Appendix A, page 131.
	 * 
	 * [1]: Dong-Ming Yan, Wenping Wang, Yang Liu, Zhouwang Yang.
	 * Variational mesh segmentation via quadric surface fitting.
	 * Elsevier: Computer-Aided Design 44 (2012), pp. 1072-1082.
	 *
	 * [2]: Hui Ma, Geometric Fitting of Quadratic Curves and Surfaces,
	 * Dissertation Thesis, Univ. Alabama, 2011.
	 */
	int CalcQuadricSurface_Taubin(const int vertexID,
	                              float (&coeffs)[10]);
	int CalcQuadricSurface_sphere(const float radius,
	                              const Vector3F& centre,
	                              float (&coeffs)[10])
	{
		const float R=radius*radius;
		coeffs[0]=-1.f + (centre.x*centre.x +centre.y*centre.y +centre.z*centre.z)/R;
		coeffs[1]=-2.f*centre.x /R;
		coeffs[2]=-2.f*centre.y /R;
		coeffs[3]=-2.f*centre.z /R;
		coeffs[4]=0.f;
		coeffs[5]=0.f;
		coeffs[6]=0.f;
		coeffs[7]=1.f/R;
		coeffs[8]=1.f/R;
		coeffs[9]=1.f/R;
		return(0);
	}

	/**
	 * Calculate the 3rd coordinate (e.g., z) to complete a 3D point
	 * [x,y,z] given the other two coordinates (e.g., x and y).
	 * The point should lay on the quadric surface given with
	 * the coefficients \e coeffs.
	 *
	 * Returns false if such point could not be determined.
	 *
	 * Note that due to symmetries in the equations, one can use
	 * the very same function to obtain x given y,z, etc.
	 *
	 * Also note that two points are actually generated, one need
	 * to examine their distance to the point around which the surface
	 * has been calculated, and take the closer one.
	 */
	bool GetPointOnQuadricSurface(const float x,const float y,
	                              float &z1, float &z2,
	                              const float (&coeffs)[10]);

	///Adjusts the input \e point to arrive at the surface given by its \e coeffs,
	///the value of \e point is, therefore, changed.
	float GetClosestPointOnQuadricSurface(Vector3F& point,
	                                      const float (&coeffs)[10]);

	///returns index of the closest from \e points to the \e point
	int ChooseClosestPoint(const std::vector<Vector3F>& points,
	                       const Vector3F& point);


	///input is filename
	///returns 0 on success, otherwise non-zero
	int ImportSTL(const char* filename);
	int ExportSTL(const char* filename);
	int ImportVTK(const char* filename);
	int ImportVTK_Volumetric(const char* filename);

	int ImportVTK_Ftree(const char* filename,bool resetMesh=false);
	std::vector<Vector3FC> fPoints;
	std::vector<int> segFromPoint; //segment description, quite ugly solution for now
	std::vector<int> segToPoint;
	std::vector<float> segFromRadius;
	std::vector<float> segToRadius;
	size_t PointsFirstOffset;

	void RenderMask(i3d::Image3d<i3d::GRAY16>& mask,const bool showTriangles=false);
	void RenderMaskB(i3d::Image3d<i3d::GRAY16>& mask);
	void RenderOneTimeTexture(const i3d::Image3d<i3d::GRAY16>& mask,
	                          i3d::Image3d<i3d::GRAY16>& texture);

	void CenterMesh(const Vector3F& newCentre);
	void ScaleMesh(const Vector3F& scale);

	//the following functions are all defined in graphics.cpp
	void displayMesh(void);
	void displayMeshEdges(void);
	void displayVertexAndNeigs(void);
	void displayQuadricSurface(void);
	void displayTEST(void);
};

#endif
